import type { HashAlgorithm } from "./types";
export declare class Sha1 implements HashAlgorithm {
    getHash: (xml: any) => string;
    getAlgorithmName: () => string;
}
export declare class Sha256 implements HashAlgorithm {
    getHash: (xml: any) => string;
    getAlgorithmName: () => string;
}
export declare class Sha512 implements HashAlgorithm {
    getHash: (xml: any) => string;
    getAlgorithmName: () => string;
}
