import type { CanonicalizationOrTransformationAlgorithm, CanonicalizationOrTransformationAlgorithmProcessOptions, CanonicalizationOrTransformAlgorithmType } from "./types";
export declare class EnvelopedSignature implements CanonicalizationOrTransformationAlgorithm {
    protected includeComments: boolean;
    constructor();
    process(node: Node, options: CanonicalizationOrTransformationAlgorithmProcessOptions): Node;
    getAlgorithmName(): CanonicalizationOrTransformAlgorithmType;
}
